/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.perf;

import java.net.URL;
import java.util.Collection;
import java.util.List;

import org.ow2.bonita.deployment.Deployment;
import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.def.majorElement.PackageDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.exception.DeploymentException;
import org.ow2.bonita.facade.exception.IllegalTaskStateException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.PackageNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.UndeletableInstanceException;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.AccessorUtil;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.Tool;
import org.ow2.novabpm.perf.PerfTestCase;


public abstract class AbstractPerftestCase implements PerfTestCase {

  protected long id;

  public long getId() {
    return this.id;
  }
  public void setId(long id) {
    this.id = id;
  }

  protected void executeTasks(RuntimeAPI runtimeAPI, QueryRuntimeAPI queryRuntimeAPI, ProcessInstanceUUID instanceUUID,
      int expected) throws InstanceNotFoundException,
  TaskNotFoundException, IllegalTaskStateException {
    int cpt = expected;
    while (cpt > 0) {
      final Collection<ActivityInstance< TaskInstance >> activities =
        queryRuntimeAPI.getTaskList(instanceUUID, TaskState.READY);
      if (activities == null || activities.isEmpty()) {
        throw new BonitaRuntimeException("Wrong number of executed tasks. Expected is "
            + expected + " and really executed is: " + (expected - cpt));
      }
      for (ActivityInstance< TaskInstance > activity : activities) {
        TaskUUID taskUUID = activity.getBody().getUUID();
        runtimeAPI.startTask(taskUUID, true);
        runtimeAPI.finishTask(taskUUID, true);
        cpt--;
      }
    }
    if (cpt < 0) {
      throw new BonitaRuntimeException("More tasks than expected(" + expected + ") are available for instance: " + instanceUUID);
    }
    try {
      final Collection<ActivityInstance< TaskInstance >> tasks =
        queryRuntimeAPI.getTaskList(instanceUUID, TaskState.READY);
      if (tasks != null && tasks.size() > 0) {
        throw new BonitaRuntimeException("More tasks than expected(" + expected
            + ") are available for instance: " + instanceUUID);
      }
    } catch (InstanceNotFoundException e) {
      // TODO: nothing, all tasks are executed and the instance may be finished
    }
  }

  protected long executeCommonInstance(int expectedTasks) {
    long endTime = -1;
    ProcessInstanceUUID instanceUUID = null;
    RuntimeAPI runtimeAPI = AccessorUtil.getAPIAccessor().getRuntimeAPI();
    QueryRuntimeAPI queryRuntimeAPI = AccessorUtil.getAPIAccessor().getQueryRuntimeAPI();
    try {
      instanceUUID = runtimeAPI.instantiateProcess(getProcessUUID());
      executeTasks(runtimeAPI, queryRuntimeAPI, instanceUUID, expectedTasks);
      endTime = System.currentTimeMillis();
    } catch (Throwable e) {
      throw new BonitaRuntimeException(e);
    } finally {
      if (instanceUUID != null) {
        try {
          runtimeAPI.deleteProcessInstance(instanceUUID);
        } catch (InstanceNotFoundException e) {
          throw new BonitaRuntimeException(e);
        } catch (UndeletableInstanceException e) {
          throw new BonitaRuntimeException(e);
        }
      }
    }
    return endTime;
  }

  public void deploy() {
    URL xpdlUrl = getXpdlUrl();
    if (xpdlUrl == null) {
      throw new BonitaRuntimeException("Can't find xpdl ressource: xpdlUrl is null");
    }
    ManagementAPI managementAPI = AccessorUtil.getAPIAccessor().getManagementAPI();
    Deployment deployment = new Deployment(xpdlUrl, Tool.getClasses(getClasses().toArray(new Class[]{})));
    try {
      managementAPI.deploy(deployment);
    } catch (DeploymentException e) {
      throw new BonitaRuntimeException("Error during deployment", e);
    }
  }

  public void undeploy() {
    try {
      ManagementAPI managementAPI = AccessorUtil.getAPIAccessor().getManagementAPI();
      managementAPI.undeploy(getPackageUUID());
      managementAPI.deletePackage(getPackageUUID());
    } catch (Exception e) {
      throw new BonitaRuntimeException("Error during undeployment", e);
    }
  }

  private PackageDefinitionUUID getPackageUUID() throws PackageNotFoundException {
    String packageId = getPackageId();
    PackageDefinition packageDefinition = AccessorUtil.getAPIAccessor().getQueryDefinitionAPI().getLastPackage(packageId);
    return packageDefinition.getUUID();
  }

  private ProcessDefinitionUUID getProcessUUID() throws PackageNotFoundException, ProcessNotFoundException {
    String processId = getProcessId();
    ProcessDefinition processDefinition = 
      AccessorUtil.getAPIAccessor().getQueryDefinitionAPI().getPackageProcess(getPackageUUID(), processId);
    return processDefinition.getUUID();
  }

  protected abstract String getPackageId();
  protected abstract URL getXpdlUrl();
  protected abstract List<Class< ? >> getClasses();
}


